@@ -159,7 +159,8 @@ public:
 		tga.colourmaptype = 0;
 		tga.imagetype = 2;
 
-		memset(tga.colormapspecs,0,5);
+		for (unsigned int i = 0; i < 5; ++i)
+			tga.colormapspecs[i] = '\0';
 		tga.xstart = (short) im.offset().x();
 		tga.ystart = (short) im.offset().y();
 		tga.height = (short) im.height();
@@ -180,10 +181,10 @@ public:
 			//is a 8-digits binary number code  
 			// always 0 0  |  mirroring | bits 
 			//(future uses)|  image     | for alpha-channel
-			//--------------------------------------------			
-			//     7 6     |      5 4   |      3 2 1 0
 			//--------------------------------------------
-			//     0 0     |      1 0   |      1 0 0 0
+			//     7 6     |     5 4    |      3 2 1 0
+			//--------------------------------------------
+			//     0 0     |     1 0    |      1 0 0 0
 			tga.descriptor = (char) 40;
 			tga.bits = (char) 32;
 		}
